import 'dart:convert';

import 'package:crypto/crypto.dart';
import 'package:json_annotation/json_annotation.dart';
import 'package:retgoo_sdk/models/variable.dart';

part 'directform.g.dart';

@JsonSerializable()
class DirectForm {
  @JsonKey(name: "IdProcessInstance")
  int idProcessInstance;

  @JsonKey(name: "IdMasterProcess")
  int idMasterProcess;

  @JsonKey(name: "MasterProcessName")
  String masterProcessName;

  @JsonKey(name: "ProcessName")
  String processName;

  @JsonKey(name: "TaskInfo")
  TaskInfo taskInfo;

  @JsonKey(name: "Diagram")
  Diagram diagram;

  @JsonKey(name: "Variable")
  BusinessData variable;

  @JsonKey(name: "BusinessInfo")
  BusinessInfo businessInfo;

  @JsonKey(name: "ProcessCreatedBy")
  ProcessCreator createdBy;

  @JsonKey(name: "ProcessCreatedAt")
  DateTime createdAt;

  DirectForm();

  factory DirectForm.fromJson(Map<String, dynamic> json) =>
      _$DirectFormFromJson(json);
  Map<String, dynamic> toJson() => _$DirectFormToJson(this);

  String get encodeTaskId {
    final sha = sha256.newInstance();
    final digest = sha
        .convert(utf8.encode("$idMasterProcess/${taskInfo?.mainForm?.taskId}"));
    return digest.toString().toUpperCase();
  }

  @override
  bool operator ==(Object other) =>
      identical(this, other) ||
      other is DirectForm &&
          runtimeType == other.runtimeType &&
          idProcessInstance == other.idProcessInstance;

  @override
  int get hashCode => idProcessInstance.hashCode;
}

@JsonSerializable()
class ProcessCreator {
  @JsonKey(name: "Id")
  int id;

  @JsonKey(name: "Name")
  String name;

  ProcessCreator();

  factory ProcessCreator.fromJson(Map<String, dynamic> json) =>
      _$ProcessCreatorFromJson(json);
  Map<String, dynamic> toJson() => _$ProcessCreatorToJson(this);
}

@JsonSerializable()
class Diagram {
  @JsonKey(name: "Bpmn20Export")
  String bpmn20Export;
  Diagram();

  factory Diagram.fromJson(Map<String, dynamic> json) =>
      _$DiagramFromJson(json);
  Map<String, dynamic> toJson() => _$DiagramToJson(this);
}

@JsonSerializable()
class TaskInfo {
  @JsonKey(name: "MainForm")
  UserFormTask mainForm;
  TaskInfo();

  factory TaskInfo.fromJson(Map<String, dynamic> json) =>
      _$TaskInfoFromJson(json);
  Map<String, dynamic> toJson() => _$TaskInfoToJson(this);
}

@JsonSerializable()
class UserFormTask {
  @JsonKey(name: "TaskId")
  String taskId;

  @JsonKey(name: "TaskName")
  String taskName;

  @JsonKey(name: "IdToken")
  int idToken;

  @JsonKey(name: "IdTokenForm")
  int idTokenForm;

  UserFormTask();

  factory UserFormTask.fromJson(Map<String, dynamic> json) =>
      _$UserFormTaskFromJson(json);
  Map<String, dynamic> toJson() => _$UserFormTaskToJson(this);
}

@JsonSerializable()
class BusinessInfo {
  @JsonKey(name: "Duration")
  int duration;

  @JsonKey(name: "Owner")
  String owner;

  @JsonKey(name: "BusinessGoal")
  String businessGoal;

  @JsonKey(name: "BusinessValue")
  BusinessValue businessValue;

  BusinessInfo();

  factory BusinessInfo.fromJson(Map<String, dynamic> json) =>
      _$BusinessInfoFromJson(json);
  Map<String, dynamic> toJson() => _$BusinessInfoToJson(this);
}

@JsonSerializable()
class BusinessValue {
  @JsonKey(name: "Shareholder")
  int shareholder;

  @JsonKey(name: "Managerial")
  int managerial;

  @JsonKey(name: "Employee")
  int employee;

  @JsonKey(name: "Social")
  int social;

  @JsonKey(name: "Customer")
  int customer;

  @JsonKey(name: "Partner")
  int partner;

  BusinessValue();

  factory BusinessValue.fromJson(Map<String, dynamic> json) =>
      _$BusinessValueFromJson(json);
  Map<String, dynamic> toJson() => _$BusinessValueToJson(this);
}
