import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/user.dart';
import 'package:retgoo_sdk/pages/home/notification_page.dart';
import 'package:retgoo_sdk/pages/home/shortcut_page.dart';
import 'package:retgoo_sdk/providers/authentication_provider.dart';
import 'package:retgoo_sdk/retgoo_sdk.dart';
import 'package:retgoo_sdk/utils/notification_utils.dart';

class HomePage extends StatefulWidget {
  HomePage({Key key}) : super(key: key);

  @override
  _HomePageState createState() => _HomePageState();
}

class _HomePageState extends State<HomePage> {
  int _currentIndex = 0;
  bool get isLarge => MediaQuery.of(context).size.width > 600;

  @override
  void initState() {
    super.initState();
    FirebaseMessaging().configure(
      onMessage: handleFirebaseMessage,
    );
  }

  @override
  Widget build(BuildContext context) {
    return AnnotatedRegion(
      value: SystemUiOverlayStyle.light.copyWith(
        statusBarColor: Colors.transparent,
      ),
      child: Scaffold(
        backgroundColor: isLarge ? null : Theme.of(context).primaryColor,
        body: isLarge ? buildDesktop() : buildMobile(),
        bottomNavigationBar: isLarge
            ? null
            : BottomNavigationBar(
                backgroundColor: Colors.white,
                elevation: 16.0,
                selectedFontSize: 12,
                unselectedFontSize: 12,
                items: [
                  BottomNavigationBarItem(
                    icon: Icon(_currentIndex == 0
                        ? RetGooIcon.proses_bisnis_selected
                        : RetGooIcon.proses_bisnis_normal),
                    label: "menu.shortcuts".tr,
                  ),
                  BottomNavigationBarItem(
                    icon: Icon(_currentIndex == 1
                        ? RetGooIcon.tugas_selected
                        : RetGooIcon.tugas_normal),
                    label: "menu.tasks".tr,
                  ),
                  BottomNavigationBarItem(
                    icon: Icon(_currentIndex == 2
                        ? RetGooIcon.profil_selected
                        : RetGooIcon.profil_normal),
                    label: "menu.profile".tr,
                  ),
                ],
                currentIndex: _currentIndex,
                onTap: (index) {
                  setState(() {
                    _currentIndex = index;
                  });
                },
              ),
      ),
    );
  }

  buildDesktop() {
    return Row(
      children: [
        NavigationRail(
          elevation: 8.0,
          leading: CircleAvatar(
            child: Icon(CupertinoIcons.person),
          ),
          destinations: [
            NavigationRailDestination(
              icon: Icon(CupertinoIcons.app),
              label: Text("menu.shortcuts".tr),
            ),
            NavigationRailDestination(
              icon: Icon(CupertinoIcons.bell),
              label: Text("menu.tasks".tr),
            ),
            NavigationRailDestination(
              icon: Icon(CupertinoIcons.settings),
              label: Text("menu.profile".tr),
            ),
          ],
          selectedIndex: _currentIndex,
          onDestinationSelected: (index) {
            setState(() {
              _currentIndex = index;
            });
          },
        ),
        Expanded(
          child: IndexedStack(
            index: _currentIndex,
            children: [
              ShortcutPage(),
              NotificationPage(),
              Container(),
            ],
          ),
        ),
      ],
    );
  }

  buildMobile() {
    return SafeArea(
      bottom: false,
      child: Selector<AuthenticationProvider, UserAccount>(
        selector: (context, provider) => provider.currentUser,
        builder: (context, user, child) {
          return NestedScrollView(
            headerSliverBuilder: (context, scrolling) => [
              SliverAppBar(
                expandedHeight: MediaQuery.of(context).size.height * 0.35,
                flexibleSpace: FlexibleSpaceBar(
                  background: Container(
                    alignment: Alignment.center,
                    child: Column(
                      mainAxisSize: MainAxisSize.min,
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Row(
                          crossAxisAlignment: CrossAxisAlignment.start,
                          mainAxisSize: MainAxisSize.min,
                          children: [
                            Padding(
                              padding: const EdgeInsets.only(
                                right: 16.0,
                                top: 8.0,
                              ),
                              child: CircleAvatar(
                                backgroundImage: MemoryImage(user.imageBytes),
                                radius: 24,
                              ),
                            ),
                            Column(
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                Text(
                                  user.fullName,
                                  style: TextStyle(
                                    fontSize: 20.0,
                                    fontWeight: FontWeight.w900,
                                    color: Colors.white,
                                  ),
                                ),
                                Text(
                                  "@${user.username}",
                                  style: TextStyle(
                                    color: Colors.white,
                                  ),
                                ),
                              ],
                            ),
                          ],
                        ),
                        const SizedBox(
                          height: 8.0,
                        ),
                        OutlineButton(
                          child: Text("btn.switch_user".tr),
                          textColor: Colors.white,
                          shape: RoundedRectangleBorder(
                            borderRadius: BorderRadius.circular(100.0),
                          ),
                          borderSide: BorderSide(
                            color: Colors.white,
                          ),
                          onPressed: () {
                            Provider.of<AuthenticationProvider>(context,
                                    listen: false)
                                .switchUser();
                          },
                        ),
                      ],
                    ),
                  ),
                ),
              ),
            ],
            body: Container(
              margin: const EdgeInsets.only(top: 8.0),
              decoration: BoxDecoration(
                color: Theme.of(context).scaffoldBackgroundColor,
                borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(24.0),
                  topRight: Radius.circular(24.0),
                ),
                boxShadow: [
                  BoxShadow(
                    offset: Offset(0, -1),
                    blurRadius: 8.0,
                    color: Colors.black.withOpacity(0.125),
                  ),
                ],
              ),
              child: ClipRRect(
                borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(24.0),
                  topRight: Radius.circular(24.0),
                ),
                child: IndexedStack(
                  index: _currentIndex,
                  children: [
                    ShortcutPage(),
                    NotificationPage(),
                    Container(),
                  ],
                ),
              ),
            ),
          );
        },
      ),
    );
  }
}
