import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/user.dart';
import 'package:retgoo_sdk/pages/login/login_user_auth.dart';
import 'package:retgoo_sdk/providers/authentication_provider.dart';

class UserAuthSelector extends StatelessWidget {
  const UserAuthSelector({Key key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: buildBody(),
    );
  }

  buildBody() {
    return NestedScrollView(
      headerSliverBuilder: (context, innerScrolled) => [
        SliverAppBar(
          backgroundColor: Colors.transparent,
          expandedHeight: 250,
          flexibleSpace: FlexibleSpaceBar(
            background: Container(
              child: AnnotatedRegion(
                value: SystemUiOverlayStyle.light.copyWith(
                  statusBarColor: Colors.transparent,
                ),
                child: CustomPaint(
                  painter: _Painter(color: Theme.of(context).primaryColor),
                  child: SafeArea(
                    child: Padding(
                      padding: const EdgeInsets.all(16.0),
                      child: Column(
                        children: [
                          CircleAvatar(
                            radius: 32.0,
                            backgroundImage: NetworkImage(
                                FirebaseAuth.instance.currentUser.photoURL),
                          ),
                          const SizedBox(
                            height: 8.0,
                          ),
                          Text(
                            "${FirebaseAuth.instance.currentUser.displayName}",
                            style: TextStyle(
                              fontSize: 18.0,
                              fontWeight: FontWeight.bold,
                              color: Colors.white,
                            ),
                          ),
                          Text(
                            FirebaseAuth.instance.currentUser.email,
                            style: TextStyle(
                              color: Colors.white,
                            ),
                          ),
                          const SizedBox(
                            height: 8.0,
                          ),
                          OutlineButton(
                            textColor: Colors.white,
                            child: Text("SIGN OUT"),
                            shape: RoundedRectangleBorder(
                              borderRadius: BorderRadius.circular(100.0),
                            ),
                            borderSide: BorderSide(
                              color: Colors.white,
                            ),
                            onPressed: () {
                              Provider.of<AuthenticationProvider>(context,
                                      listen: false)
                                  .logout();
                            },
                          ),
                          Spacer(),
                          Text(
                            "Choose your account to continue",
                            style: TextStyle(),
                          )
                        ],
                      ),
                    ),
                  ),
                ),
              ),
            ),
          ),
        )
      ],
      body: AnnotatedRegion(
        value: SystemUiOverlayStyle.dark.copyWith(
          statusBarColor: Colors.transparent,
        ),
        child: StreamBuilder<QuerySnapshot>(
          stream: FirebaseFirestore.instance
              .collection(
                  "users/${FirebaseAuth.instance.currentUser.uid}/portals")
              .snapshots(),
          builder: (context, snapshot) {
            if (snapshot.connectionState == ConnectionState.waiting) {
              return Center(
                child: CircularProgressIndicator(),
              );
            }

            final items = snapshot.data.docs;
            return ListView.builder(
              padding: const EdgeInsets.all(16.0),
              itemCount: items.length + 1,
              itemBuilder: (context, index) {
                if (index < items.length) {
                  final item = items.elementAt(index);
                  return Card(
                    child: ListTile(
                      leading: Icon(CupertinoIcons.person_circle),
                      trailing: IconButton(
                        icon: Icon(CupertinoIcons.delete),
                        onPressed: () {
                          item.reference.delete();
                        },
                      ),
                      title: Text(item["nick"]),
                      subtitle: Text("@${item["username"]}"),
                      onTap: () {
                        Provider.of<AuthenticationProvider>(context,
                                listen: false)
                            .setUser(UserAccount()
                              ..accessToken = item["access_token"]);
                      },
                    ),
                  );
                }

                return ListTile(
                  leading: Icon(CupertinoIcons.add),
                  title: Text("Add New Account"),
                  subtitle: Text("Sign into another account"),
                  onTap: () {
                    Navigator.push(
                      context,
                      MaterialPageRoute(
                        builder: (context) => UserAuthPage(),
                      ),
                    );
                  },
                );
              },
            );
          },
        ),
      ),
    );
  }
}

class _Painter extends CustomPainter {
  final Color color;
  _Painter({this.color = Colors.indigo});

  @override
  void paint(Canvas canvas, Size size) {
    double height = size.height - 24.0;
    Path p = Path()
      ..moveTo(0, 0)
      ..lineTo(0, height * 0.8)
      ..quadraticBezierTo(
        size.width / 2,
        height,
        size.width,
        height * 0.8,
      )
      ..lineTo(size.width, 0.0)
      ..close();

    canvas.drawPath(
      p,
      Paint()
        ..style = PaintingStyle.fill
        ..color = color,
    );
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) => false;
}
