import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/models/directform.dart';
import 'package:retgoo_sdk/providers/process_provider.dart';
import 'package:retgoo_sdk/retgoo_sdk.dart';

class ProcessInformationPage extends StatelessWidget {
  ProcessInformationPage({Key key}) : super(key: key);

  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("process.info.title".tr),
      ),
      body: SizedBox.expand(
        child: Selector<ProcessProvider, DirectForm>(
          selector: (context, provider) => provider.directForm,
          builder: (context, directForm, child) {
            return buildData(directForm);
          },
        ),
      ),
    );
  }

  buildCard({String section, Map<String, dynamic> data}) {
    return Card(
      child: Padding(
        padding: const EdgeInsets.all(16.0),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: [
            buildSection(section),
            KeyValues(data: data),
          ],
        ),
      ),
    );
  }

  buildData(DirectForm directForm) {
    return SingleChildScrollView(
      padding: const EdgeInsets.all(8.0),
      physics: AlwaysScrollableScrollPhysics(),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: [
          buildCard(
            section: "process.info.lbl.business_process".tr,
            data: {
              "process.info.name".tr: directForm.masterProcessName,
              "process.info.version".tr: directForm.processName,
            },
          ),
          buildCard(
            section: "process.info.lbl.execution".tr,
            data: {
              "process.info.process_id".tr:
                  NumberFormat("#,##0", "def.languageCode".tr)
                      .format(directForm.idProcessInstance),
              "process.info.created_by".tr:
                  directForm.createdBy?.name ?? "Undefined",
              "process.info.created_at".tr:
                  DateFormat("EEEE, dd MMMM yyyy HH:mm", "def.languageCode".tr)
                      .format(directForm?.createdAt?.toLocal()),
              "process.info.time_limit".tr: "Tidak terbatas",
              "process.info.last_action".tr: "none",
              "process.info.status".tr: "Ready",
            },
          ),
          buildCard(
            section: "process.info.lbl.business_challenge".tr,
            data: {
              "process.info.goal".tr:
                  directForm.businessInfo.businessGoal ?? "undefined",
              "process.info.owner".tr:
                  directForm.businessInfo.owner ?? "undefined",
              "process.info.participants".tr:
                  directForm.businessInfo.owner ?? "undefined",
              "process.info.duration".tr:
                  directForm.businessInfo.owner ?? "undefined",
            },
          ),
          buildCard(
            section: "process.info.lbl.business_values".tr,
            data: {
              "process.info.shareholder".tr:
                  directForm.businessInfo.businessValue.shareholder,
              "process.info.managerial".tr:
                  directForm.businessInfo.businessValue.managerial,
              "process.info.partner".tr:
                  directForm.businessInfo.businessValue.partner,
              "process.info.customer".tr:
                  directForm.businessInfo.businessValue.customer,
              "process.info.employee".tr:
                  directForm.businessInfo.businessValue.employee,
              "process.info.social".tr:
                  directForm.businessInfo.businessValue.social,
            },
          ),
        ],
      ),
    );
  }

  buildSection(String caption) {
    return Padding(
      padding: const EdgeInsets.symmetric(vertical: 8.0),
      child: Row(
        children: [
          Icon(
            CupertinoIcons.pin_fill,
            size: 18.0,
          ),
          const SizedBox(
            width: 8.0,
          ),
          Expanded(
            child: Text(
              caption,
              style: TextStyle(
                fontSize: 18.0,
                fontWeight: FontWeight.bold,
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class KeyValues extends StatelessWidget {
  final Map<String, dynamic> data;
  const KeyValues({Key key, @required this.data}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Table(
      columnWidths: {
        0: FixedColumnWidth(150),
        1: FixedColumnWidth(8.0),
      },
      children: [
        for (var key in data.keys)
          TableRow(
            children: [
              Text(
                key,
                style: TextStyle(
                  fontSize: 14.0,
                  fontWeight: FontWeight.w300,
                ),
              ),
              Text(
                ":",
                style: TextStyle(
                  fontSize: 14.0,
                  fontWeight: FontWeight.w300,
                ),
              ),
              Text(
                data[key].toString(),
                style: TextStyle(
                  fontSize: 14.0,
                ),
              ),
            ],
          ),
      ],
    );
  }
}
