import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_zoom_drawer/flutter_zoom_drawer.dart';
import 'package:provider/provider.dart';
import 'package:retgoo_sdk/forms/forms.dart';
import 'package:retgoo_sdk/models/directform.dart';
import 'package:retgoo_sdk/pages/process/loading_page.dart';
import 'package:retgoo_sdk/providers/process_provider.dart';
import 'package:retgoo_sdk/widgets/retgoo_server.dart';
import 'package:retgoo_sdk/retgoo_sdk.dart';

import 'completed_page.dart';
import 'diagram_page.dart';
import 'not_implemented_page.dart';
import 'process_information_page.dart';

int currentPID;

class ProcessPage extends StatefulWidget {
  final int idProcessInstance;
  final int idActivityNotification;

  ProcessPage({Key key, this.idActivityNotification, this.idProcessInstance})
      : super(key: key);

  @override
  _ProcessPageState createState() => _ProcessPageState();
}

class _ProcessPageState extends State<ProcessPage> {
  ProcessProvider provider;

  @override
  void initState() {
    super.initState();
    provider = ProcessProvider(
      idProcessInstance: widget.idProcessInstance,
      idNotificationActivity: widget.idActivityNotification,
    );
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    RetGooServer.of(context).setListener("update_client_token", (value) {
      provider.refreshToken();
    });

    RetGooServer.of(context).setListener("task_token_completed", (value) {
      provider.refreshToken();
    });

    RetGooServer.of(context).setListener("activity_token_list", (value) {
      provider.refreshActivities();
    });
  }

  @override
  deactivate() {
    RetGooServer.of(context).removeListener("task_token_completed");
    RetGooServer.of(context).removeListener("activity_token_list");
    RetGooServer.of(context).removeListener("update_client_token");
    super.deactivate();
  }

  Widget build(BuildContext context) {
    return ChangeNotifierProvider.value(
      value: provider,
      child: Scaffold(
        body: ProcessContent(),
      ),
    );
  }
}

class ProcessContent extends StatefulWidget {
  const ProcessContent({
    Key key,
  }) : super(key: key);

  @override
  _ProcessContentState createState() => _ProcessContentState();
}

class _ProcessContentState extends State<ProcessContent> {
  DirectForm directForm;
  Offset offset;

  int _currentIndex = 0;
  double _sideButtonSize = 72;

  final zoomController = ZoomDrawerController();
  @override
  void initState() {
    super.initState();
    Future.delayed(Duration(milliseconds: 500), () {
      zoomController.stateNotifier?.addListener(() {
        setState(() {});
      });
    });
  }

  @override
  void dispose() {
    currentPID = null;
    super.dispose();
  }

  buildBody() {
    return AnnotatedRegion(
      value: SystemUiOverlayStyle.dark.copyWith(
        statusBarColor: Colors.transparent,
      ),
      child: ZoomDrawer(
        controller: zoomController,
        menuScreen: buildMenu(),
        mainScreen: buildMain(),
        showShadow: true,
        borderRadius: 48.0,
        openCurve: Curves.easeOutBack,
        closeCurve: Curves.easeOutBack,
        slideWidth: MediaQuery.of(context).size.width / 4 * 3,
        angle: -5.0,
      ),
    );
  }

  openMenu() {
    zoomController.open();
  }

  closeMenu() {
    zoomController.close();
  }

  Widget buildMain() {
    return IgnorePointer(
      ignoring: [DrawerState.open, DrawerState.opening]
              .contains(zoomController?.stateNotifier?.value) ??
          false,
      child: WillPopScope(
        onWillPop: () async {
          if (zoomController.isOpen()) {
            return true;
          }

          openMenu();
          return false;
        },
        child: Stack(
          children: [
            Positioned.fill(
              child: IndexedStack(
                index: _currentIndex,
                children: [
                  buildForm(),
                  ProcessInformationPage(),
                  DiagramPage(),
                ],
              ),
            ),
            Positioned(
              top: offset?.dy ??
                  (MediaQuery.of(context).size.height - _sideButtonSize) / 2,
              right: 0,
              child: _SideButton(
                size: _sideButtonSize,
                onMoved: (offset) {
                  setState(() {
                    this.offset = offset;
                  });
                },
                onTap: openMenu,
              ),
            ),
          ],
        ),
      ),
    );
  }

  buildMenu() {
    return AnnotatedRegion(
      value: SystemUiOverlayStyle.light.copyWith(
        statusBarColor: Colors.transparent,
      ),
      child: DefaultTextStyle(
        style: DefaultTextStyle.of(context).style.copyWith(
              color: Colors.white,
            ),
        child: Container(
          decoration: BoxDecoration(
            gradient: LinearGradient(
              colors: [
                Theme.of(context).primaryColor,
                Theme.of(context).accentColor,
              ],
              begin: Alignment.topLeft,
              end: Alignment.bottomRight,
            ),
          ),
          child: SafeArea(
            child: Padding(
              padding: const EdgeInsets.symmetric(horizontal: 8.0),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 16.0),
                    child: Text(
                      directForm.masterProcessName,
                      style: TextStyle(
                        fontSize: 26.0,
                        fontWeight: FontWeight.bold,
                      ),
                    ),
                  ),
                  const SizedBox(
                    height: 24.0,
                  ),
                  buildMenuButton(
                    selected: _currentIndex == 0,
                    label: "menu.form".tr,
                    icon: CupertinoIcons.doc_on_clipboard,
                    onTap: () {
                      setState(() {
                        _currentIndex = 0;
                      });
                      closeMenu();
                    },
                  ),
                  buildMenuButton(
                    selected: _currentIndex == 1,
                    label: "menu.information".tr,
                    icon: CupertinoIcons.info,
                    onTap: () {
                      closeMenu();
                      setState(() {
                        _currentIndex = 1;
                      });
                    },
                  ),
                  buildMenuButton(
                    selected: _currentIndex == 2,
                    label: "menu.diagram".tr,
                    icon: CupertinoIcons.flowchart,
                    onTap: () {
                      setState(() {
                        _currentIndex = 2;
                      });
                      closeMenu();
                    },
                  ),
                  const SizedBox(
                    height: 24.0,
                  ),
                  buildMenuButton(
                    selected: false,
                    label: "menu.close".tr,
                    icon: CupertinoIcons.chevron_back,
                    onTap: () {
                      Navigator.pop(context);
                    },
                  ),
                ],
              ),
            ),
          ),
        ),
      ),
    );
  }

  buildMenuButton({
    String label,
    IconData icon,
    bool selected,
    GestureTapCallback onTap,
  }) {
    return Material(
      borderRadius: BorderRadius.circular(100.0),
      clipBehavior: Clip.antiAliasWithSaveLayer,
      color: selected ? Colors.black.withOpacity(0.125) : Colors.transparent,
      textStyle: TextStyle(
        color: Colors.white,
      ),
      child: InkWell(
        onTap: onTap,
        child: Padding(
          padding: const EdgeInsets.all(16.0),
          child: Row(
            children: [
              Icon(
                icon,
                color: Colors.white,
              ),
              const SizedBox(
                width: 16.0,
              ),
              Text(
                label,
                style: TextStyle(
                  fontSize: 16.0,
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  var userTask;
  buildForm() {
    return Selector<ProcessProvider, ProcessState>(
      selector: (context, provider) => provider.state,
      builder: (context, state, child) {
        if (state == ProcessState.completed) {
          return CompletedPage();
        }

        final directForm =
            Provider.of<ProcessProvider>(context, listen: false).directForm;
        final registrationId = directForm.encodeTaskId;
        currentPID = directForm.idProcessInstance;

        if (!registeredUserTasks.containsKey(registrationId)) {
          return NotImplementedPage(directForm: directForm);
        }

        if (userTask == null) {
          userTask = registeredUserTasks[registrationId]();
        }

        return userTask.attach(context);
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Selector<ProcessProvider, DirectForm>(
      selector: (context, provider) => provider.directForm,
      builder: (context, directForm, child) {
        if (directForm == null) {
          return LoadingPage();
        }

        this.directForm = directForm;
        return Scaffold(
          body: buildBody(),
        );
      },
    );
  }
}

class _SideButton extends StatefulWidget {
  final double size;
  final ValueChanged<Offset> onMoved;
  final GestureTapCallback onTap;
  const _SideButton({
    Key key,
    this.size = 64,
    this.onMoved,
    this.onTap,
  }) : super(key: key);

  @override
  _SideButtonState createState() => _SideButtonState();
}

class _SideButtonState extends State<_SideButton>
    with TickerProviderStateMixin {
  final buttonKey = GlobalKey();

  @override
  Widget build(BuildContext context) {
    return AnimatedOpacity(
      opacity: 0.5,
      duration: Duration(milliseconds: 200),
      child: Draggable(
        affinity: Axis.vertical,
        axis: Axis.vertical,
        childWhenDragging: Container(),
        onDragEnd: (details) {
          double y = details.offset.dy;
          if (y < 0) {
            y = 0;
          }

          if (y > MediaQuery.of(context).size.height - widget.size) {
            y = MediaQuery.of(context).size.height - widget.size;
          }

          final offset = Offset(details.offset.dx, y);
          if (widget.onMoved != null) {
            widget.onMoved(offset);
          }
        },
        feedback: buildSideMenu(),
        child: GestureDetector(
          onTap: widget.onTap,
          child: buildSideMenu(),
        ),
      ),
    );
  }

  Widget buildSideMenu() {
    return Container(
      key: buttonKey,
      height: widget.size,
      width: widget.size / 2,
      decoration: BoxDecoration(
        color: Colors.black.withOpacity(0.5),
        borderRadius: BorderRadius.only(
          topLeft: Radius.circular(widget.size * 2),
          bottomLeft: Radius.circular(widget.size * 2),
        ),
      ),
      padding: const EdgeInsets.all(4.0),
      child: Padding(
        padding: const EdgeInsets.all(8.0),
        child: Icon(
          CupertinoIcons.fullscreen_exit,
          color: Colors.white,
        ),
      ),
    );
  }
}
