import 'dart:convert';
import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter_local_notifications/flutter_local_notifications.dart';
import 'package:retgoo_sdk/pages/process/process_page.dart';
import 'package:retgoo_sdk/plugins/retgoo_sdk_channel.dart';
import 'package:retgoo_sdk/widgets/retgoo_app.dart';

Future<dynamic> handleFirebaseMessage(Map<String, dynamic> payload) async {
  var data;
  var notification;

  if (Platform.isIOS) {
    data = payload;
    notification = data;
  }

  if (Platform.isAndroid) {
    data = payload["data"];
    notification = payload["notification"];
  }

  final title = notification["title"];
  final body = notification["body"];

  final type = data["type"];
  if (type != null && type == "direct_form") {
    final value = json.decode(data["value"]);
    if (value != null) {
      final pid = int.tryParse(value["id_process_instance"]);
      if (currentPID != pid) {
        makeNotification(
          id: pid,
          title: title,
          body: body,
          payload: json.encode(data),
        );
      }
    }
  }
}

final FlutterLocalNotificationsPlugin flutterLocalNotificationsPlugin =
    FlutterLocalNotificationsPlugin();
initializeNotification() async {
  // final NotificationAppLaunchDetails notificationAppLaunchDetails =
  //     await flutterLocalNotificationsPlugin.getNotificationAppLaunchDetails();

  const AndroidInitializationSettings initializationSettingsAndroid =
      AndroidInitializationSettings('ic_stat_retgoo');

  /// Note: permissions aren't requested here just to demonstrate that can be
  /// done later
  final IOSInitializationSettings initializationSettingsIOS =
      IOSInitializationSettings(
    requestAlertPermission: true,
    requestBadgePermission: true,
    requestSoundPermission: true,
    onDidReceiveLocalNotification: didReceiveLocalNotificationSubject,
  );

  const MacOSInitializationSettings initializationSettingsMacOS =
      MacOSInitializationSettings(
          requestAlertPermission: false,
          requestBadgePermission: false,
          requestSoundPermission: false);

  final InitializationSettings initializationSettings = InitializationSettings(
    android: initializationSettingsAndroid,
    iOS: initializationSettingsIOS,
    macOS: initializationSettingsMacOS,
  );

  await flutterLocalNotificationsPlugin.initialize(
    initializationSettings,
    onSelectNotification: onSelectNotification,
  );
}

Future<dynamic> didReceiveLocalNotificationSubject(
  int id,
  String title,
  String body,
  String payload,
) async {}

Future<dynamic> onSelectNotification(String payload) async {
  RetGooSdkChannel.clearIntent();

  final data = json.decode(payload);

  final type = data["type"];
  if (type == "direct_form") {
    final value = json.decode(data["value"]);
    final idActivity = int.tryParse(value["id_notification_activity"]);
    final idProcessInstace = int.tryParse(value["id_process_instance"]);
    navigatorKey.currentState?.push(
      MaterialPageRoute(
        builder: (context) => ProcessPage(
          idProcessInstance: idProcessInstace,
          idActivityNotification: idActivity,
        ),
      ),
    );

    return;
  }
}

Future<void> makeNotification({
  int id,
  String title,
  String body,
  String payload,
}) async {
  const androidPlatformChannelSpecifics = AndroidNotificationDetails(
    'DEFAULT',
    'Umum',
    'Notifikasi Tugas',
    importance: Importance.max,
    priority: Priority.high,
    ticker: 'ticker',
  );

  const iOSPlatformChannelSpecifics = IOSNotificationDetails();

  const NotificationDetails platformChannelSpecifics = NotificationDetails(
    android: androidPlatformChannelSpecifics,
    iOS: iOSPlatformChannelSpecifics,
  );

  await flutterLocalNotificationsPlugin.show(
    id,
    title,
    body,
    platformChannelSpecifics,
    payload: payload,
  );
}
